#!/usr/bin/perl -w

use Data::Dumper;
use File::stat;

sub check_and_update_cache {
	my $cachefile = shift;
	my $servername = shift;
	my $nmondir = shift;
	my $ssh_idfile = shift;
	my $debug = shift;
	my $nmon_runmode = shift;

	$nmon_runmode = "daemon" if($nmon_runmode ne "daemon" || $nmon_runmode ne "user");

	#if file older than 5 minutes
	if(-e $cachefile) {
		if( (time - stat($cachefile)->mtime) > 280 ) {
			print "cachefile too old, updating...\n" if($debug > 1);
			&update_nmon_cache_user($cachefile,$servername,$nmondir,$ssh_idfile,$debug) if($nmon_runmode eq "user");
			&update_nmon_cache_daemon($cachefile,$servername,$nmondir,$ssh_idfile,$debug) if($nmon_runmode eq "daemon");
		}
		if( stat($cachefile)->size < 2048) {
			print "cachefile too small, updating...\n" if($debug > 1);
			&update_nmon_cache_user($cachefile,$servername,$nmondir,$ssh_idfile,$debug) if($nmon_runmode eq "user");
			&update_nmon_cache_daemon($cachefile,$servername,$nmondir,$ssh_idfile,$debug) if($nmon_runmode eq "daemon");
		}
	}
	
	else {
		print "cachefile does not exist, updating...\n" if($debug > 1);
		&update_nmon_cache_user($cachefile,$servername,$nmondir,$ssh_idfile,$debug) if($nmon_runmode eq "user");
		&update_nmon_cache_daemon($cachefile,$servername,$nmondir,$ssh_idfile,$debug) if($nmon_runmode eq "daemon");
	}
}

sub update_nmon_cache_daemon {
	my $cachefile = shift;
	my $servername = shift;
	my $nmondir = shift;
	my $ssh_idfile = shift;
	my $debug = shift;
	my $nmon_tmpdir = shift;

	my $sshcommand = qq(ssh -o StrictHostKeyChecking=no -i $ssh_idfile nagprobe\@$servername);

	my $command = q('cd /tmp/nmon; NMONFILE=`ls -tr -1 /tmp/nmon |tail -1`; tail -1 $NMONFILE |awk -F , "{ print \$3 }" |xargs -I% grep % $NMONFILE |awk -F , "{print \$1}" > /tmp/nmon.pattern; grep -f /tmp/nmon.pattern $NMONFILE |grep -v ,T; rm /tmp/nmon.pattern');
	my $fullcommand = $sshcommand . " " . $command . "> $cachefile";
	print "FULLCOMMAND: $fullcommand\n" if($debug > 1);
	`$fullcommand`;

	$command = q('cd /tmp/nmon; NMONFILE=`ls -tr -1 /tmp/nmon |tail -1`; tail -1 $NMONFILE |awk -F , "{ print \$3 }" |xargs -I% grep % $NMONFILE');
	$fullcommand = $sshcommand . " " . $command . ">> $cachefile";
	print "FULLCOMMAND: $fullcommand\n" if($debug > 1);
	`$fullcommand`;
}

sub update_nmon_cache_user {
        my $cachefile = shift;
        my $servername = shift;
        my $nmondir = shift;
        my $ssh_idfile = shift;
        my $debug = shift;

        my $nmon_binary = &query_nmon_binary($servername,$ssh_idfile,$debug);

        &check_nmon_fifo($servername,$ssh_idfile,$debug);

        my $command = "ssh -o StrictHostKeyChecking=no -i $ssh_idfile nagprobe\@$servername '$nmondir$nmon_binary -s 2 -c 1 -F /tmp/nmonfifo & cat /tmp/nmonfifo' > $cachefile";
        print "$command\n" if($debug > 1);
        `$command`;
}

sub check_nmon_fifo {
	my $servername = shift;
	my $ssh_idfile = shift;
	my $debug = shift;
	print "checking existence of nmon fifo and creating it if necessary...\n" if($debug > 1);
	my $result = `ssh -o StrictHostKeyChecking=no -i $ssh_idfile nagprobe\@$servername 'if [ ! -e /tmp/nmonfifo ]; then mkfifo /tmp/nmonfifo; fi'`;
}

sub query_nmon_binary {
	my $servername = shift;
	my $ssh_idfile = shift;
	my $debug = shift;

	my $oslevel = `ssh -o StrictHostKeyChecking=no -i $ssh_idfile nagprobe\@$servername '/usr/bin/oslevel'`;
	print "$oslevel\n" if($debug > 1);

	my $nmon_binary = "";

	$nmon_binary = "nmon"		if($oslevel =~ /^7.1/);
	$nmon_binary = "nmon12e_aix61"	if($oslevel =~ /^6.1/);
	$nmon_binary = "nmon_aix53"	if($oslevel =~ /^5.3/);
	$nmon_binary = "nmon_aix52"	if($oslevel =~ /^5.2/);
	$nmon_binary = "nmon12e_aix51"	if($oslevel =~ /^5.1/);
	$nmon_binary = "nmon_aix433"	if($oslevel =~ /^4.3.3/);
	$nmon_binary = "nmon_aix432"	if($oslevel =~ /^4.3.2/);
	$nmon_binary = "nmon_aix420"	if($oslevel =~ /^4.2/);
	$nmon_binary = "nmon_aix415"	if($oslevel =~ /^4.1/);

	if($nmon_binary eq "nmon_aix52") {
		my $ml_oslevel = `ssh -o StrictHostKeyChecking=no -i $ssh_idfile nagprobe\@$servername '/usr/bin/oslevel -r | cut -c 6,7'`;

		$nmon_binary = "nmon12e_aix522"		if($ml_oslevel =~ /01/);
		$nmon_binary = "nmon12e_aix522"		if($ml_oslevel =~ /02/);
		## fall back to the older nmon release for these 2 problem releases
		$nmon_binary = "nmon12e_aix51"		if($ml_oslevel =~ /03/);
		$nmon_binary = "nmon12e_aix51"		if($ml_oslevel =~ /04/);
		$nmon_binary = "nmon12e_aix527"		if($ml_oslevel =~ /05/);
		$nmon_binary = "nmon12e_aix527"		if($ml_oslevel =~ /06/);
		$nmon_binary = "nmon12e_aix527"		if($ml_oslevel =~ /07/);
		$nmon_binary = "nmon12e_aix527"		if($ml_oslevel =~ /08/);
		$nmon_binary = "nmon12e_aix527"		if($ml_oslevel =~ //);
	}
	
	if($nmon_binary eq "nmon_aix53") {
		my $ml_oslevel = `ssh -o StrictHostKeyChecking=no -i $ssh_idfile nagprobe\@$servername '/usr/bin/oslevel -r | cut -c 6,7'`;

		$nmon_binary = "nmon12e_aix530"		if($ml_oslevel =~ /00/);
		$nmon_binary = "nmon12e_aix530"		if($ml_oslevel =~ /01/);
		$nmon_binary = "nmon12e_aix530"		if($ml_oslevel =~ /02/);
		$nmon_binary = "nmon12e_aix530"		if($ml_oslevel =~ /03/);
		$nmon_binary = "nmon12e_aix534"		if($ml_oslevel =~ /04/);
		$nmon_binary = "nmon12e_aix535"		if($ml_oslevel =~ /05/);
		$nmon_binary = "nmon12e_aix536"		if($ml_oslevel =~ /06/);
		$nmon_binary = "nmon12e_aix537"		if($ml_oslevel =~ /07/);
	}
	return $nmon_binary;
}

1;
