#!/usr/bin/perl -w

use Data::Dumper;
use File::stat;
use lib "/srv/www/htdocs/cacti/scripts";
use query_nmon;

if ( (!exists($ARGV[0])) || (($ARGV[1] ne "query") && ($ARGV[1] ne "get") && ($ARGV[1] ne "index") && ($ARGV[1] ne "num_indexes")) ) {
	print "usage:\n\n";
	print "./query_nmon_cpustats.pl <hostname> index\n";
	print "./query_nmon_cpustats.pl <hostname> num_indexes\n";
	print "./query_nmon_cpustats.pl <hostname> query {device,user,system,wait,idle}\n";
	print "./query_nmon_cpustats.pl <hostname> get {device,user,system,wait,idle} DEVICE\n";
	exit;
}

my $debug = 0;

my $servername = $ARGV[0];
my $command = $ARGV[1];
my $attribute = $ARGV[2] if exists($ARGV[2]);
my $device = $ARGV[3] if exists($ARGV[3]);

my $nmondir = "/usr/net/bin/nmon/";
my $cachedir = "/var/cache/cacti/";
my $cachefile = $cachedir . $servername;

my $ssh_idfile = "/home/cacti/.ssh/id_rsa";

my $cpustats = {};


check_and_update_cache($cachefile,$servername,$nmondir,$ssh_idfile,$debug);

open(NMONFILE, $cachefile);

foreach $line (<NMONFILE>) {
        if (my ($cpuid,$values_str) = $line =~ /^(CPU\d+),T\d{4},(.+)$/) {
		#CPU01,CPU 1 azh-550t-1,User%,Sys%,Wait%,Idle%
		#CPU01,T0001,10.0,21.1,1.6,67.3
		#CPU02,T0001,2.0,3.7,0.0,94.3
		#CPU03,T0001,2.9,12.6,0.0,84.6
		#CPU04,T0001,1.0,0.0,0.0,99.0

		$n_elements = keys(%$cpustats);
		print "cpuid: $cpuid\n" if($debug > 1);
		print "n_elements: $n_elements\n" if($debug > 1);
			
		$cpustats->{$n_elements}->{device} = $cpuid;
		
		my @values = split(/,/,$values_str);
		$cpustats->{$n_elements}->{cpu_user} = $values[0];
		$cpustats->{$n_elements}->{cpu_system} = $values[1];
		$cpustats->{$n_elements}->{cpu_wait} = $values[2];
		$cpustats->{$n_elements}->{cpu_idle} = $values[3];
	}
}

print Dumper($cpustats) if($debug > 1);

if($command eq "index") {
	foreach $key (sort keys %$cpustats) {
		print scalar $key;
		print "\n";
	}
}
elsif($command eq "query") {
	foreach $key (sort keys %$cpustats) {
		print scalar $key;
		print ":" . $cpustats->{$key}->{$attribute} . " \n";

	}
}
elsif($command eq "get") {
	print $cpustats->{$device}->{$attribute};
	print "\n";
}
elsif($command eq "num_indexes") {
	print scalar keys %$cpustats;
	print "\n";
}
