#!/usr/bin/perl -w

use Data::Dumper;
use File::stat;
use lib "/srv/www/htdocs/cacti/scripts";
use query_nmon;

if ( (!exists($ARGV[0])) || (($ARGV[1] ne "query") && ($ARGV[1] ne "get") && ($ARGV[1] ne "index") && ($ARGV[1] ne "num_indexes")) ) {
	print "usage:\n\n";
	print "./query_nmon_diskstats.pl <hostname> index\n";
	print "./query_nmon_diskstats.pl <hostname> num_indexes\n";
	print "./query_nmon_diskstats.pl <hostname> query {device,busy,read,write,xfer,bsize}\n";
	print "./query_nmon_diskstats.pl <hostname> get {device,busy,read,write,xfer,bsize} DEVICE\n";
	exit;
}

my $debug = 0;

my $servername = $ARGV[0];
my $command = $ARGV[1];
my $attribute = $ARGV[2] if exists($ARGV[2]);
my $device = $ARGV[3] if exists($ARGV[3]);

my $nmondir = "/usr/net/bin/nmon/";
my $cachedir = "/var/cache/cacti/";
my $cachefile = $cachedir . $servername;

my $ssh_idfile = "/home/cacti/.ssh/id_rsa";

my $diskstats = {};


check_and_update_cache($cachefile,$servername,$nmondir,$ssh_idfile,$debug);

open(NMONFILE, $cachefile);

foreach $line (<NMONFILE>) {
        if (my ($type,$description,$disks_str) = $line =~ /^(DISKBUSY),(Disk\s[^,]+),(.+)$/) {
                #DISKBUSY,Disk %Busy alp2a21,hdisk11,hdisk12,hdisk13,hdisk14,hdisk15,hdisk16,hdisk17,hdisk0
		#
		#get available harddisks and prefill hash

		my @disks = split(/,/,$disks_str);

		my $i=0;
		foreach $disk (@disks) {
			$diskstats->{$i}->{device} = $disk;
			$i++;
		}
		
	}
        if (my ($type,$values_str) = $line =~ /^DISK(\w+),T\d{4},(.+)$/) {
                #DISKBUSY,Disk %Busy alp2a21,hdisk11,hdisk12,hdisk13,hdisk14,hdisk15,hdisk16,hdisk17,hdisk0
                #DISKREAD,Disk Read KB/s alp2a21,hdisk11,hdisk12,hdisk13,hdisk14,hdisk15,hdisk16,hdisk17,hdisk0
                #DISKWRITE,Disk Write KB/s alp2a21,hdisk11,hdisk12,hdisk13,hdisk14,hdisk15,hdisk16,hdisk17,hdisk0
                #DISKXFER,Disk transfers per second alp2a21,hdisk11,hdisk12,hdisk13,hdisk14,hdisk15,hdisk16,hdisk17,hdisk0
                #DISKBSIZE,Disk Block Size alp2a21,hdisk11,hdisk12,hdisk13,hdisk14,hdisk15,hdisk16,hdisk17,hdisk0
                #DISKBUSY,T0001,0.2,0.0,0.0,0.0,0.0,0.0,0.0,0.0
                #DISKREAD,T0001,237.7,72.5,72.5,72.5,0.1,0.1,0.1,181.2
                #DISKWRITE,T0001,42.3,3.1,0.1,9.9,0.0,0.0,0.1,0.0
                #DISKXFER,T0001,19.6,1.4,0.6,2.1,0.2,0.2,0.2,1.5
                #DISKBSIZE,T0001,14.3,54.9,113.9,39.2,0.5,0.5,0.8,116.9

		my @values = split(/,/,$values_str);
		$type = lc $type;
		my $i=0;
		foreach $value (@values) {
			$diskstats->{$i}->{$type} = $value;
			$i++;
		}
	}
}

print Dumper($diskstats) if($debug > 1);

if($command eq "index") {
	foreach $key (sort keys %$diskstats) {
		print scalar $key;
		print "\n";
		#$diskstats->{$key}->{name} . "\n";

	}
}
elsif($command eq "query") {
	foreach $key (sort keys %$diskstats) {
		if(exists $diskstats->{$key}->{$attribute}) {
			print scalar $key;
			print ":" . $diskstats->{$key}->{$attribute} . " \n";
		}
		else {
			print scalar $key;
			print ":0\n";
		}

	}
}
elsif($command eq "get") {
	print $diskstats->{$device}->{$attribute};
	print "\n";
}
elsif($command eq "num_indexes") {
	print scalar keys %$diskstats;
	print "\n";
}
#print Dumper $diskstats;
