#!/usr/bin/perl -w

use Data::Dumper;
use File::stat;
use lib "/srv/www/htdocs/cacti/scripts";
use query_nmon;

if ( (!exists($ARGV[0])) || (($ARGV[1] ne "query") && ($ARGV[1] ne "get") && ($ARGV[1] ne "index") && ($ARGV[1] ne "num_indexes")) ) {
	print "usage:\n\n";
	print "./query_nmon_diskstats.pl <hostname> index\n";
	print "./query_nmon_diskstats.pl <hostname> num_indexes\n";
	print "./query_nmon_diskstats.pl <hostname> query {device,busy,read,write,xfer,bsize}\n";
	print "./query_nmon_diskstats.pl <hostname> get {device,busy,read,write,xfer,bsize} DEVICE\n";
	exit;
}

my $debug = 0;

my $servername = $ARGV[0];
my $command = $ARGV[1];
my $attribute = $ARGV[2] if exists($ARGV[2]);
my $device = $ARGV[3] if exists($ARGV[3]);

my $nmondir = "/usr/net/bin/nmon/";
my $cachedir = "/var/cache/cacti/";
my $cachefile = $cachedir . $servername;

my $ssh_idfile = "/home/cacti/.ssh/id_rsa";

my $stats = {};


check_and_update_cache($cachefile,$servername,$nmondir,$ssh_idfile,$debug);

open(NMONFILE, $cachefile);

my $helper = {};

foreach $line (<NMONFILE>) {
        if (my ($type,$description,$attrs_str) = $line =~ /^(IOADAPT\w*),(Disk\s[^,]+),(.+)$/) {
		#IOADAPT,Disk Adapter azh-550t-1,vscsi1_read-KB/s,vscsi1_write-KB/s,vscsi1_xfer-tps,vscsi0_read-KB/s,vscsi0_write-KB/s,vscsi0_xfer-tps

		my @attrs = split(/,/,$attrs_str);

		my $i=0;
		foreach $attr (@attrs) {

			$helper->{$type}->{$i} = $attr;

			print "device: $type - i: $i - attribute: $attr\n" if($debug > 1);

			$i++;
		}
		
	}
        if (my ($type,$values_str) = $line =~ /^(IOADAPT\w*),T\d{4},(.+)$/) {
		#IOADAPT,T0001,0.0,0.0,0.0,1192.4,138.4,26.8

		print "$line\n" if($debug > 1);


		my @values = split(/,/,$values_str);

		$i=0;
		$j=0;
		foreach $value (@values) {
			print "helper string: $helper->{$type}->{$i}\n" if($debug > 1);
			my ($device,$attribute) = $helper->{$type}->{$i} =~ /(\w+)_(.+)/;

			print "device: $device - attribute: $attribute\n" if($debug > 1);
			
			my $exists=0;
			my $exists_id=0;
			foreach $key (keys %$stats) {
				if($stats->{$key}->{device} eq $device) {
					print "$device already exists, not adding new entry to hash\n" if($debug > 1);
					$exists = 1;
					$exists_id = $key;
					last;
				}
			}

			if($exists == 1) {
				$stats->{$exists_id}->{$attribute} = $value;
			}
			else {
				$stats->{$j}->{device} = $device;
				$stats->{$j}->{$attribute} = $value;
				$j++;
			}
			$i++;
		}
	}
}


print Dumper($stats) if($debug > 1);

if($command eq "index") {
	foreach $key (sort keys %$stats) {
		print scalar $key;
		print "\n";
	}
}
elsif($command eq "query") {
	foreach $key (sort keys %$stats) {
		print scalar $key;
		print ":" . $stats->{$key}->{$attribute} . " \n";

	}
}
elsif($command eq "get") {
	print $stats->{$device}->{$attribute};
	print "\n";
}
elsif($command eq "num_indexes") {
	print scalar keys %$stats;
	print "\n";
}
#print Dumper $diskstats;
