#!/usr/bin/perl -w

use Data::Dumper;
use File::stat;
use lib "/srv/www/htdocs/cacti/scripts";
use query_nmon;

if ( (!exists($ARGV[0])) || (($ARGV[1] ne "query") && ($ARGV[1] ne "get") && ($ARGV[1] ne "index") && ($ARGV[1] ne "num_indexes")) ) {
	print "usage:\n\n";
	print "./query_nmon_jfsstats.pl <hostname> index\n";
	print "./query_nmon_jfsstats.pl <hostname> num_indexes\n";
	print "./query_nmon_jfsstats.pl <hostname> query {mountpoint,jfsfile,jfsinode}\n";
	print "./query_nmon_jfsstats.pl <hostname> get {mountpoint,jfsfile,jfsinode} DEVICE\n";
	exit;
}

my $debug = 0;

my $servername = $ARGV[0];
my $command = $ARGV[1];
my $attribute = $ARGV[2] if exists($ARGV[2]);
my $device = $ARGV[3] if exists($ARGV[3]);

my $nmondir = "/usr/net/bin/nmon/";
my $cachedir = "/var/cache/cacti/";
my $cachefile = $cachedir . $servername;
my $ssh_idfile = "/home/cacti/.ssh/id_rsa";
my $runmode = "daemon";

my $stats = {};


check_and_update_cache($cachefile,$servername,$nmondir,$ssh_idfile,$debug,$runmode);

open(NMONFILE, $cachefile);

foreach $line (<NMONFILE>) {
        if (my ($type,$description,$attributes_str) = $line =~ /^(JFSFILE),(JFS\s[^,]+),(.+)$/) {
		#JFSFILE,JFS Filespace %Used azh-550t-1,/,/home,/usr,/var,/tmp,/opt,/usr/net,/admin,/nfstest,/nfs3test
		#JFSINODE,JFS Inode %Used azh-550t-1,/,/home,/usr,/var,/tmp,/opt,/usr/net,/admin,/nfstest,/nfs3test
		#
		#get available harddisks and prefill hash

		my @attributes = split(/,/,$attributes_str);

		my $i=0;
		foreach $attribute (@attributes) {
			$stats->{$i}->{mountpoint} = $attribute;
			$i++;
		}
		
	}
        if (my ($type,$values_str) = $line =~ /^(JFS\w+),T\d{4},(.+)$/) {
		#JFSFILE,T0001,86.7,57.0,84.3,37.6,27.3,72.8,82.1,0.3,0.4,0.4
		#JFSINODE,T0001,34.0,29.7,24.3,6.9,0.5,19.1,42.2,0.0,0.0,0.0

		my @values = split(/,/,$values_str);
		$type = lc $type;
		my $i=0;
		foreach $value (@values) {
			$stats->{$i}->{$type} = $value;
			$i++;
		}
	}
}

print Dumper($stats) if($debug > 1);

if($command eq "index") {
	foreach $key (sort keys %$stats) {
		print scalar $key;
		print "\n";
	}
}
elsif($command eq "query") {
	foreach $key (sort keys %$stats) {
		if(exists $stats->{$key}->{$attribute}) {
			print scalar $key;
			print ":" . $stats->{$key}->{$attribute} . " \n";
		}
		else {
			print scalar $key;
			print ":0\n";
		}

	}
}
elsif($command eq "get") {
	if(exists $stats->{$device}->{$attribute}) {
		print $stats->{$device}->{$attribute};
		print "\n";
	}
	else {
		print "0\n";
	}
}
elsif($command eq "num_indexes") {
	print scalar keys %$stats;
	print "\n";
}
