#!/usr/bin/perl -w

use Data::Dumper;
use File::stat;
use lib "/srv/www/htdocs/cacti/scripts";
use query_nmon;

if ( (!exists($ARGV[0])) || (($ARGV[1] ne "query") && ($ARGV[1] ne "get") && ($ARGV[1] ne "index") && ($ARGV[1] ne "num_indexes")) ) {
	print "usage:\n\n";
	print "./query_nmon_diskstats.pl <hostname> index\n";
	print "./query_nmon_diskstats.pl <hostname> num_indexes\n";
	print "./query_nmon_diskstats.pl <hostname> query {device,busy,read,write,xfer,bsize}\n";
	print "./query_nmon_diskstats.pl <hostname> get {device,busy,read,write,xfer,bsize} DEVICE\n";
	exit;
}

my $debug = 0;

my $servername = $ARGV[0];
my $command = $ARGV[1];
my $attribute = $ARGV[2] if exists($ARGV[2]);
my $device = $ARGV[3] if exists($ARGV[3]);

my $nmondir = "/usr/net/bin/nmon/";
my $cachedir = "/var/cache/cacti/";
my $cachefile = $cachedir . $servername;
my $ssh_idfile = "/home/cacti/.ssh/id_rsa";
my $nmon_runmode = "daemon";

my $stats = {};


check_and_update_cache($cachefile,$servername,$nmondir,$ssh_idfile,$debug,$nmon_runmode);

open(NMONFILE, $cachefile);

my $helper = {};

foreach $line (<NMONFILE>) {
        if (my ($type,$description,$attrs_str) = $line =~ /^(NET\w*),(Network\s[^,]+),(.+)$/) {
		#NET,Network I/O azh-550t-1,en6-read-KB/s,en7-read-KB/s,en8-read-KB/s,lo0-read-KB/s,en6-write-KB/s,en7-write-KB/s,en8-write-KB/s,lo0-write-KB/s
		#NETPACKET,Network Packets azh-550t-1,en6-reads/s,en7-reads/s,en8-reads/s,lo0-reads/s,en6-writes/s,en7-writes/s,en8-writes/s,lo0-writes/s
		#NETSIZE,Network Size azh-550t-1,en6-readsize,en7-readsize,en8-readsize,lo0-readsize,en6-writesize,en7-writesize,en8-writesize,lo0-writesize
		#NETERROR,Network Errors azh-550t-1,en6-ierrs,en7-ierrs,en8-ierrs,lo0-ierrs,en6-oerrs,en7-oerrs,en8-oerrs,lo0-oerrs,en6-collisions,en7-collisions,en8-collisions,lo0-collisions
		#
		#get available netifs and prefill hash

		my @attrs = split(/,/,$attrs_str);

		my $i=0;
		foreach $attr (@attrs) {

			$helper->{$type}->{$i} = $attr;

			print "device: $type - i: $i - attribute: $attr\n" if($debug > 1);

			$i++;
		}
		
	}
        if (my ($type,$values_str) = $line =~ /^(NET\w*),T\d{4},(.+)$/) {
		#NET,T0001,1.5,0.1,0.0,0.2,3.7,0.1,0.0,0.2
		#NETPACKET,T0001,5.7,1.3,0.0,2.7,4.1,0.7,0.0,2.7
		#NETSIZE,T0001,276.9,46.6,0.0,59.2,930.7,86.0,0.0,59.2
		#NETERROR,T0001,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0

		print "$line\n" if($debug > 1);


		my @values = split(/,/,$values_str);

		$i=0;
		$j=0;
		foreach $value (@values) {
			print "helper string: $helper->{$type}->{$i}\n" if($debug > 1);
			my ($device,$attribute) = $helper->{$type}->{$i} =~ /([\w]{2,5})-(.+)/;

			print "device: $device - attribute: $attribute\n" if($debug > 1);
			
			my $exists=0;
			my $exists_id=0;
			foreach $key (keys %$stats) {
				if($stats->{$key}->{device} eq $device) {
					print "$device already exists, not adding new entry to hash\n" if($debug > 1);
					$exists = 1;
					$exists_id = $key;
					last;
				}
			}

			if($exists == 1) {
				$stats->{$exists_id}->{$attribute} = $value;
			}
			else {
				$stats->{$j}->{device} = $device;
				$stats->{$j}->{$attribute} = $value;
				$j++;
			}
			$i++;
		}
	}
}


print Dumper($stats) if($debug > 1);

if($command eq "index") {
	foreach $key (sort keys %$stats) {
		print scalar $key;
		print "\n";
	}
}
elsif($command eq "query") {
	foreach $key (sort keys %$stats) {
		if(exists $stats->{$key}->{$attribute}) {
			print scalar $key;
			print ":" . $stats->{$key}->{$attribute} . " \n";
		}
		else {
			print scalar $key;
			print ":0\n";
		}

	}
}
elsif($command eq "get") {
	if(exists $stats->{$device}->{$attribute}) {
		print $stats->{$device}->{$attribute};
		print "\n";
	}
	else {
		print "0\n";
	}
}
elsif($command eq "num_indexes") {
	print scalar keys %$stats;
	print "\n";
}
#print Dumper $diskstats;
